var next_data_url; // 次のコンテンツのURL
var prev_data_url; // 前のコンテンツのURL
var next_data_cache; // 次のコンテンツのキャッシュ
var prev_data_cache; // 前のコンテンツのキャッシュ
var last_scroll = 0;
var is_loading = false; // ローディンの判定
var headerHeight = $('.header').height();

// 初期設定
function init() {
	var $itemList = $('.js-item-list');
	next_data_url = $itemList.data('next-url');
	prev_data_url = $itemList.data('prev-url');

	$(window).scroll(function() { scrollFn() });
	// コンテンツよりwindowの方が高かった時
	if ($(window).height() > $(".js-scroll-content").height()) {
		if (next_data_url !== "") {
			loadNext();
		}
	}
	primeCache();
};

// スクロール時に実行
function scrollFn() {
	// スクロール位置を更新
	var scroll_pos = $(window).scrollTop();

	if (scroll_pos >= 0.9 * ($(document).height() - $(window).height())) {
		if (!is_loading) loadNext();
	}
	if (scroll_pos <= 0.9 * $("#header").height()) {
		if (!is_loading) loadPrev();
	}
	// URLを変えるタイミング
	if (Math.abs(scroll_pos - last_scroll) > $(window).height() * 0.1) {
		last_scroll = scroll_pos;
		$('.js-item-list').each(function() {
			if (mostlyVisible(this)) {
				history.replaceState(null, null, $(this).attr("data-url"));
				return;
			}
		});
	}
};

// 次のコンテンツをプリロード
function loadNext() {
	if (next_data_url !== "") {
		is_loading = true;

		function showNext(data) {
			$('.js-item-list-wrap').append(data);
			next_data_url = data.data('next-url');
			next_data_cache = false;
			$.ajax({
				type: 'GET',
				url: next_data_url,
				dataType: 'html',
				success: function (data) {
					var $itemList = $(data).find('.js-item-list');
					next_data_cache = $itemList;
				},
				error: function () {
					alert('通信に失敗しました');
				}
			});
		};

		if (next_data_cache) {
			showNext(next_data_cache);
			is_loading = false;
		} else {
			$.ajax({
				type: 'GET',
				url: next_data_url,
				dataType: 'html',
				success: function (data) {
					var $itemList = $(data).find('.js-item-list');
					next_data_cache = $itemList;
					showNext($itemList);
					is_loading = false;
				},
				error: function () {
					alert('通信に失敗しました');
				}
			});
		}
	}
};

// 前のコンテンツをプリロード
function loadPrev() {
	if (prev_data_url !== "") {
		is_loading = true;

		function showPrevious(data) {
			$('.js-item-list-wrap').prepend(data);
			item_height = $(".js-item-list:first").height();
			window.scrollTo(0, $(window).scrollTop() + item_height + headerHeight); // adjust scroll
			prev_data_url = data.data('prev-url');
			prev_data_cache = false;
			$.ajax({
				type: 'GET',
				url: prev_data_url,
				dataType: 'html',
				success: function(data) {
					var $itemList = $(data).find('.js-item-list');
					prev_data_cache = $itemList;
				},
				error: function() {
					alert('通信に失敗しました');
				}
			});
		}

		if (prev_data_cache) {
			showPrevious(prev_data_cache);
			is_loading = false;
		} else {
			$.ajax({
				type: 'GET',
				url: prev_data_url,
				dataType: 'html',
				success: function(data) {
					var $itemList = $(data).find('.js-item-list');
					showPrevious($itemList);
					is_loading = false;
				},
				error: function() {
					alert('通信に失敗しました');
				}
			});
		}
	}
};

// URLが切り替わるタイミングを判定
function mostlyVisible(element) {
	var scroll_pos = $(window).scrollTop();
	var window_height = $(window).height();
	var el_top = $(element).offset().top;
	var el_height = $(element).height();
	var el_bottom = el_top + el_height;
	return ((el_bottom - el_height * 0.25 > scroll_pos) &&
	(el_top < (scroll_pos + 0.5 * window_height)));
};

// HTMLが読み込まれたタイミングでAjax通信をおこなう
// 取得したコンテンツをキャッシュする
function primeCache() {
	var $loading = $('.loading');

	$.ajax({
		type: 'GET',
		url: next_data_url,
		dataType: 'html',
		success: function (data) {
			$loading.show();
			var $itemList = $(data).find('.js-item-list');
			next_data_cache = $itemList;
		},
		complete: function() {
			$loading.hide();
		},
		error: function () {
			$loading.hide();
			alert('通信に失敗しました');
		}
	});
	$.ajax({
		type: 'GET',
		url: prev_data_url,
		dataType: 'html',
		success: function (data) {
			var $itemList = $(data).find('.js-item-list');
			prev_data_cache = $itemList;
		},
		error: function () {
			alert('通信に失敗しました');
		}
	});
}

